resource "aws_lb" "default" {
  load_balancer_type = "application"
  name               = "${var.name}-${var.env}-alb"
  subnets            = local.subnet_ids
  idle_timeout       = var.idle_timeout

  security_groups = [
    aws_security_group.alb.id,
  ]

  enable_deletion_protection = true

  tags = {
    Name = "${var.name}-${var.env}-alb"
    Env  = var.env
  }

  lifecycle {
    ignore_changes = [
      tags,
    ]
  }
}

resource "aws_lb_listener" "default" {
  for_each = local.listeners

  load_balancer_arn = aws_lb.default.arn
  port              = each.value[0].alb_port
  protocol          = each.value[0].alb_schema
  ssl_policy        = each.value[0].is_secure ? "ELBSecurityPolicy-2016-08" : null
  certificate_arn   = each.value[0].is_secure ? local.ssl_cert_arn : null

  # If no .host specified – default to forward
  dynamic "default_action" {
    for_each = length(compact([for i,z in each.value: i if z.host != ""])) == 0 ? [1] : []
    content {
      type             = "forward"
      target_group_arn = aws_lb_target_group.default[each.value[0].app_port].arn
    }
  }

  # If .host specified – default to 404 (because rest is targeted with rules)
  dynamic "default_action" {
    for_each = length(compact([for i,z in each.value: i if z.host != ""])) > 0 ? [1] : []
    content {
      type             = "fixed-response"
      fixed_response {
          content_type = "text/plain"
          status_code  = "404"
          message_body = "Not Found"
      }
    }
  }
}

resource "aws_lb_listener" "redirect80to443" {
  count = var.redirect_80_to_443 == true ? 1 : 0

  load_balancer_arn = aws_lb.default.arn
  port              = 80
  protocol          = "HTTP"

  default_action {
    type = "redirect"
    redirect {
      port        = "443"
      protocol    = "HTTPS"
      status_code = "HTTP_301"
    }
  }
}

resource "aws_lb_listener_rule" "https" {
  for_each = local.https_with_hosts

  listener_arn = aws_lb_listener.default["443"].arn

  action {
    order            = each.value.order
    type             = "forward"
    target_group_arn = aws_lb_target_group.default[each.value.app_port].arn
  }

  condition {
    host_header {
      values = each.value.hosts
    }
  }
}

resource "aws_lb_target_group" "default" {
  for_each = local.target_groups

  name     = "${var.name}-${var.env}-${each.key}-tg"
  vpc_id   = var.jz_vpc.self.id

  port     = each.value[0].app_port
  protocol = each.value[0].app_schema

  health_check {
      enabled             = true
      path                = each.value[0].healthcheck_path
      healthy_threshold   = 2
      unhealthy_threshold = 5
      interval            = 30
      timeout             = 5
      matcher             = "200-499"
  }
}

resource "aws_lb_target_group_attachment" "default" {
  for_each = {
    for item in local.tg_x_instance: "${item.lb_tg_port}-${item.lb_tg_protocol}-${item.instance_id}" => item
  }

  target_group_arn = each.value.lb_tg_arn
  target_id        = each.value.instance_id
}

# Default cert
data "aws_acm_certificate" "default" {
  count       = var.ssl_cert_domain != "" ? 1 : 0
  domain      = var.ssl_cert_domain
  most_recent = true
}

# Additional certs
resource "aws_lb_listener_certificate" "additional" {
  for_each        = local.ssl_cert_arns
  listener_arn    = aws_lb_listener.default["443"].arn
  certificate_arn = each.value.ssl_cert_arn
}
